import React, { useState } from 'react';
import Info from './Info';
import * as Drawing from '@mindfusion/drawing';
import * as Diagramming from '@mindfusion/diagramming';
import * as Graphs from '@mindfusion/graphs';
import { DiagramView } from '@mindfusion/diagramming-react';

const brushes = ["#9caac6", "#ce0000", "#c0c0c0"];

const onRandomTree = (diagram) => {
  diagram.clearAll();

  // create the root node
  var rect = new Drawing.Rect(0, 0, 15, 10);
  var node = new Diagramming.ShapeNode(diagram);
  var brush = brushes[0];
  node.bounds = rect;
  node.brush = brush;
  node.stroke = brushes[1];
  diagram.addItem(node);

  // add random children
  randomTree(node, 5, 4);

  applyLayout(diagram);
}

const randomTree = (node, depth, minChildren) => {
  if (depth <= 0)
    return;

  var diagram = node.parent;
  var children = randomInt(depth) - 1 + minChildren;
  var brush = brushes[0];
  while (brush === node.brush)
    brush = brushes[1];

  if (diagram.nodes.length < 3 && children < 2)
    children = 2;

  for (var i = 0; i < children; ++i) {
    // create child node
    var rect = new Drawing.Rect(0, 0, 15, 10);
    var child = new Diagramming.ShapeNode(diagram);
    child.bounds = rect;
    child.brush = brush;
    diagram.addItem(child);

    // create a link
    var link = new Diagramming.DiagramLink(diagram, node, child);
    diagram.addItem(link);

    // build child branch
    randomTree(child, depth - 1, minChildren);
  }
}

const applyLayout = (diagram) => {
  var layout = new Graphs.FractalLayout();
  layout.root = diagram.nodes[0];
  diagram.arrange(layout);
  fitItems(diagram);
}

const randomInt = (max) => {
  return Math.floor(max * Math.random());
}

const fitItems = (diagram) => {
  var rect = Drawing.Rect.empty;
  diagram.nodes.forEach(function (node) {
    if (rect === Drawing.Rect.empty)
      rect = node.bounds;
    else
      rect = rect.union(node.bounds);
  });

  if (rect)
    diagram.bounds = (new Drawing.Rect(0, 0, rect.right() + 10, rect.bottom() + 10));
}


function FractalLayout() {
  const [diagram] = useState(new Diagramming.Diagram());
  const [zoomFactor, setZoomFactor] = useState(8);

  const onZoomIn = () => {
    if (zoomFactor > 200) return;
    setZoomFactor(zoomFactor + 10);
  }

  const onZoomOut = () => {
    if (zoomFactor < 9) return;
    setZoomFactor(zoomFactor - 10);
  }

  const props = {
    "linkHeadShapeSize": 2,
    "behavior": Diagramming.Behavior.Pan,
    "zoomFactor": zoomFactor
  };

  return (
    <div className="App">
      <div className="header">
        <button onClick={() => onRandomTree(diagram)}>Random tree</button>
        <button onClick={() => onZoomIn()}>Zoom in</button>
        <button onClick={() => onZoomOut()}>Zoom out</button>
      </div>

      <div className="container">
        <div className="main">
          <DiagramView diagram={diagram} {...props} />
        </div>
        <div className="sidebar">
          <h1>About this sample</h1>
          <p>Click the button to create a random tree and arrange it using the FractalLayout algorithm.</p>
          <Info />
        </div>
      </div>

      <div className="footer">
        <p>Copyright 2024 MindFusion LLC.</p>
      </div>

    </div>
  );
}

export default FractalLayout;
